<?xml version="1.0" ?>
<!-- word2007.xsl : Copyright 2006-2015 Duxbury Systems, Inc.  All Rights Reserved. -->
<!-- embed <xsl:message>Text</xsl:message> to get traces through xmlproc/xslt.cpp/dxxmlTransformerXSLStep::OnDebugXSL -->
<!-- Namespaces for output document:
     dbtml is for XHTML-like content that forms the backbone of the document
     dbt is for DBT-specific tweaks in the output stream. -->
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:func="http://exslt.org/functions"
                xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math"
                xmlns:o="urn:schemas-microsoft-com:office:office"
                xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
                xmlns:v="urn:schemas-microsoft-com:vml"
                xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
                xmlns:w14="http://schemas.microsoft.com/office/word/2010/wordml"
                xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing"
                xmlns:rp="http://schemas.openxmlformats.org/package/2006/relationships"
                xmlns:dbt="http://www.duxburysystems.com/dbt"
                xmlns:dbtfunc="http://www.duxburysystems.com/xsl"
		xmlns="http://www.duxburysystems.com/dbtml"
                extension-element-prefixes="func dbtfunc"
                exclude-result-prefixes="m o r rp v w wp">

  <!-- Parameters: These are inputs to the XSL code. Normally, these are passed by DBT.
                   When testing this XSL code with xsltproc, they can be passed as command-line
                   parameters.
                   All are passed, and are subsequently tested, as string values.
		   The values listed below are simply default values. The defaults are set up
                   to be the same what DBT would pass with default user preference settings.
                   Having default settings allows us to run the XSL code, for example using
                   xsltproc, without having to supply values for every required parameter.
                   Importantly, it also allows us to use our latest XSL code with older versions
                   of DBT, provided that the default value for a parameter is what a newer DBT
                   would supply by default. -->

  <!-- braille: 'true'/'false' : import braille and simbraille font text as brlinline or brldisplay -->
  <xsl:param name="braille" select="'false'"/>
  <!-- cbc: 'true'/'false' : import text with Courier in the font name as compinline or compdisplay -->
  <xsl:param name="cbc" select="'false'"/>
  <!-- dbt_version (new in DBT 11.2) : MajorMinorSP version of DBT, as a number (e.g. 110200) -->
  <xsl:param name="dbt_version" select="110100"/>
  <!-- defer_endnotes: 'true'/'false' (new in DBT 11.2) : import endnotes at the end of the document -->
  <xsl:param name="defer_endnotes" select="'true'"/>
  <!-- defer_endnotes: 'true'/'false' (new in DBT 11.2) : import footnotes following the ref paragraph -->
  <xsl:param name="defer_footnotes" select="'true'"/>
  <!-- ignore_bold: 'true'/'false' : ignore bold attribute -->
  <xsl:param name="ignore_bold" select="'false'"/>
  <!-- ignore_italic: 'true'/'false' : ignore italic attribute -->
  <xsl:param name="ignore_italics" select="'false'"/>
  <!-- ignore_lang: 'true'/'false' : ignore language attribute of text -->
  <xsl:param name="ignore_lang" select="'true'"/>
  <!-- ignore_underline: 'true'/'false' : ignore underline attribute -->
  <xsl:param name="ignore_underline" select="'false'"/>
  <!-- ignore_typeforms: 'true'/'false' : ignore typeforms -->
  <xsl:param name="ignore_typeforms" select="'false'"/>
  <!-- phpb: 'true'/'false' : preserve hard page breaks -->
  <xsl:param name="phpb" select="'true'"/>
  <!-- print_pages: 'scanned'/'internal' : treat hard page breaks as print page breaks (iff 'scanned') -->
  <xsl:param name="print_pages" select="'internal'"/>
  <!-- psl: 'true'/'false' : preserve skipped lines -->
  <xsl:param name="psl" select="'false'"/>

  <!-- rels global variable caches load of XML file word/_rels/document.xml.rels -->
  <!-- only hyperlinks logic uses this cached copy right now. othes should -->
  <xsl:variable name="rels" select="document('word/_rels/document.xml.rels')"/>

  <func:function name="func:abstractnum">
    <xsl:param name="numdoc"/>
    <xsl:param name="num"/>
    <func:result select="$numdoc/w:num[@w:numId = $num]/w:abstractNumId/@w:val"/>
  </func:function>

  <func:function name="func:reldoc">
    <xsl:param name="ridParam"/>
    <func:result select="concat('word/', document('word/_rels/document.xml.rels')/rp:Relationships/rp:Relationship[@Id = $ridParam]/@Target)"/>
  </func:function>

  <!-- see http://www.openxmldeveloper.org/forums/thread/1664.aspx for a
       description of the odd way that section breaks are handled in OpenXML -->
  <func:function name="func:getsect">
    <xsl:choose>
      <xsl:when test="w:pPr/w:sectPr">
        <func:result select="w:pPr/w:sectPr"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="nstry" select="following-sibling::*[w:pPr/w:sectPr][1]/w:pPr/w:sectPr"/>
        <xsl:choose>
          <xsl:when test="$nstry">
            <func:result select="$nstry"/>
          </xsl:when>
          <xsl:otherwise>
            <func:result select="following-sibling::*[self::w:sectPr]"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <!-- Placeholder functions for Script and Transcriber Typeform Indicator mappings -->
  <func:function name="func:ftss">
    <func:result select="''"/>
  </func:function>
  <func:function name="func:fts1">
    <func:result select="''"/>
  </func:function>
  <func:function name="func:fts2">
    <func:result select="''"/>
  </func:function>
  <func:function name="func:fts3">
    <func:result select="''"/>
  </func:function>
  <func:function name="func:fts4">
    <func:result select="''"/>
  </func:function>
  <func:function name="func:fts5">
    <func:result select="''"/>
  </func:function>

  <func:function name="func:font">
    <xsl:param name="typeface"/>
    <!-- This is handled in the combineruns step -->
    <func:result select="''"/>
  </func:function>

  <func:function name="func:fontContains">
    <xsl:param name="typeface"/>
    <!-- This is handled in the combineruns step -->
    <func:result select="''"/>
  </func:function>

  <func:function name="func:highlight">
    <xsl:param name="color"/>
    <xsl:choose>
      <xsl:when test="not(w:rPr/w:highlight)"/>
      <xsl:when test="not(w:rPr/w:highlight/@w:val)"/>
      <xsl:when test="not(w:rPr/w:highlight/@w:val = $color)"/>
      <xsl:otherwise>
        <func:result select="'true'"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <func:function name="func:textColor">
    <!-- color must be a hex RRGGBB string, e.g. FF0000 for "red" -->
    <xsl:param name="color"/>
    <xsl:choose>
      <xsl:when test="not(w:rPr/w:color)"/>
      <xsl:when test="not(w:rPr/w:color/@w:val)"/>
      <xsl:when test="not(w:rPr/w:color/@w:val = $color)"/>
      <xsl:otherwise>
        <func:result select="'true'"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <func:function name="func:dropShadow">
    <!-- Actually matches any "shadow" -->
    <xsl:choose>
      <xsl:when test="not(w:rPr/w14:shadow)"/>
      <xsl:otherwise>
        <func:result select="'true'"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <func:function name="func:outline">
    <!-- Matches any "textOutline" -->
    <xsl:choose>
      <xsl:when test="not(w:rPr/w14:textOutline)"/>
      <xsl:otherwise>
        <func:result select="'true'"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <func:function name="func:enabled">
    <xsl:param name="element"/>
    <xsl:choose>
      <xsl:when test="not($element)"/>
      <xsl:when test="$element/@w:val = 0"/>
      <xsl:otherwise>
        <func:result select="'true'"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <func:function name="func:disabled">
    <xsl:param name="element"/>
    <xsl:choose>
      <xsl:when test="not($element)"/>
      <xsl:when test="not($element/@w:val = 0)"/>
      <xsl:otherwise>
        <func:result select="'true'"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <xsl:variable name="styles" select="document('word/styles.xml')/w:styles"/>
  <func:function name="func:numPr-style">
    <xsl:param name="stynam"/>
    <xsl:variable name="style" select="$styles/w:style[@w:styleId = $stynam]"/>
    <xsl:choose>
      <xsl:when test="$style">
	<xsl:choose>
	  <xsl:when test="$style/w:pPr/w:numPr">
	    <!-- Style is numbered. -->
	    <func:result select="$style/w:pPr/w:numPr"/>
	  </xsl:when>
	  <xsl:when test="$style/w:basedOn">
	    <!-- Style is based on another style, so we recurse to find whether the base style is numbered. -->
	    <func:result select="func:numPr-style($style/w:basedOn/@w:val)"/>
	  </xsl:when>
	  <!-- [xsl:otherwise] style is not numbered. -->
	</xsl:choose>
      </xsl:when>
      <!-- [xsl:otherwise] style is not defined. -->
    </xsl:choose>
  </func:function>

  <func:function name="func:numPr">
    <xsl:choose>
      <xsl:when test="w:pPr/w:numPr">
        <func:result select="w:pPr/w:numPr"/>
      </xsl:when>
      <xsl:when test="w:pPr/w:pStyle">
        <func:result select="func:numPr-style(w:pPr/w:pStyle/@w:val)"/>
      </xsl:when>
    </xsl:choose>
  </func:function>

  <func:function name="func:numId-style">
    <xsl:param name="stynam"/>
    <xsl:variable name="style" select="$styles/w:style[@w:styleId = $stynam]"/>
    <xsl:choose>
      <xsl:when test="$style">
	<xsl:choose>
	  <xsl:when test="$style/w:pPr/w:numPr/w:numId">
	    <!-- Style is numbered. -->
	    <func:result select="$style/w:pPr/w:numPr/w:numId/@w:val"/>
	  </xsl:when>
	  <xsl:when test="$style/w:basedOn">
	    <!-- Style is based on another style, so we recurse to find whether the base style is numbered. -->
	    <func:result select="func:numId-style($style/w:basedOn/@w:val)"/>
	  </xsl:when>
	  <!-- [xsl:otherwise] style is not numbered. -->
	</xsl:choose>
      </xsl:when>
      <!-- [xsl:otherwise] style is not defined. -->
    </xsl:choose>
  </func:function>

  <func:function name="func:numId">
    <xsl:choose>
      <xsl:when test="w:pPr/w:numPr/w:numId">
        <func:result select="w:pPr/w:numPr/w:numId/@w:val"/>
      </xsl:when>
      <xsl:when test="w:pPr/w:pStyle">
        <func:result select="func:numId-style(w:pPr/w:pStyle/@w:val)"/>
      </xsl:when>
    </xsl:choose>
  </func:function>

  <func:function name="func:get-val">
    <xsl:param name="val"/>
    <xsl:param name="default"/>
    <xsl:choose>
      <xsl:when test="$val">
        <func:result select="$val"/>
      </xsl:when>
      <xsl:otherwise>
        <func:result select="$default"/>
      </xsl:otherwise>
    </xsl:choose>
  </func:function>

  <xsl:template match="w:body">
    <doc>
      <head>
	<!-- <styles-debug><xsl:copy-of select="$styles"/></styles-debug> -->
	<styles>
	  <xsl:for-each select="$styles/w:style[@w:type='paragraph' or @w:type='character']">
	    <style>
	      <xsl:attribute name="id"><xsl:value-of select="@w:styleId"/></xsl:attribute>
	      <xsl:if test="w:rPr/w:rFonts/@w:ascii">
		<xsl:attribute name="font"><xsl:value-of select="w:rPr/w:rFonts/@w:ascii"/></xsl:attribute>
	      </xsl:if>
	      <!-- Attributes (bold/italic/underline/subscript/superscript -->
	      <xsl:call-template name="add-attributes"/>
	    </style>
	  </xsl:for-each>
	</styles>
      </head>
    <body>
      <xsl:choose>
        <xsl:when test="w:p/m:oMathPara|w:p/w:r/w:object/o:OLEObject[@ProgID = 'Equation.DSMT4' or @ProgID = 'Equation.3']|w:p/m:oMath">
          <dbt:tcs/>
          <xsl:call-template name="start"/>
          <xsl:apply-templates/>
          <xsl:call-template name="endnotes"/>
          <dbt:tce/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="start"/>
          <xsl:apply-templates/>
          <xsl:call-template name="endnotes"/>
        </xsl:otherwise>
      </xsl:choose>
    </body>
    </doc>
  </xsl:template>

  <xsl:template name="start">
    <!-- Code to be run at the start of document processing would go here. -->
  </xsl:template>

  <xsl:template match="m:oMath">
    <xsl:text xml:space="preserve"> </xsl:text>
    <dbt:ts/>
    <xsl:apply-templates/>
    <dbt:te/>
  </xsl:template>

  <!-- ECMA 376 22.1.2.24 : Mathematics delimiters -->
  <xsl:template match="m:d">
    <xsl:choose>
      <xsl:when test="m:begChr/@m:val">
	<xsl:value-of select="m:begChr/@m:val"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>(</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
    <xsl:choose>
      <xsl:when test="m:endChr/@m:val">
	<xsl:value-of select="m:endChr/@m:val"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>)</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="w:p">
    <xsl:choose>
      <xsl:when test="preceding-sibling::*[1]/w:pPr/w:sectPr or not(preceding-sibling::*[1])">
        <xsl:call-template name="sectPr">
          <xsl:with-param name="sect" select="func:getsect()"/>
          <xsl:with-param name="pgbreak" select="preceding-sibling::w:p"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="w:r[1]/w:br[@w:type = 'page']">
        <xsl:call-template name="simple-hardpage"/>
      </xsl:when>
    </xsl:choose>
    <!-- Handle special cases where <p>...</p> should not be generated  -->
    <!-- <xsl:apply-templates select="w:r/w:pict/v:shape/v:textbox"/> -->
    <!-- <xsl:apply-templates select="w:r/w:drawing"/> -->
    <xsl:choose>
      <xsl:when test="parent::w:tc">
        <xsl:if test="preceding-sibling::w:p and w:r">
          <dbt:l/>
        </xsl:if>
        <xsl:apply-templates/>
      </xsl:when>
      <xsl:when test="w:r[w:t] or *[w:r/w:t] or w:smartTag or w:r/w:object/o:OLEObject[@ProgID = 'Equation.DSMT4' or @ProgID = 'Equation.3'] or m:oMath">
	<p xmlns="http://www.duxburysystems.com/dbtml" xmlns:dbt="http://www.duxburysystems.com/dbt">
          <xsl:if test="w:pPr/w:jc">
            <xsl:attribute name="jc"><xsl:value-of select="w:pPr/w:jc/@w:val"/></xsl:attribute>
          </xsl:if>
	  <xsl:if test="w:pPr/w:pStyle">
	    <xsl:attribute name="class"><xsl:value-of select="w:pPr/w:pStyle/@w:val"/></xsl:attribute>
	  </xsl:if>
          <xsl:variable name="numPr" select="func:numPr()"/>
          <xsl:if test="$numPr">
            <xsl:element name="dbt:num">
              <xsl:variable name="ilvl" select="func:get-val($numPr/w:ilvl/@w:val, '0')"/>
              <xsl:variable name="numid" select="func:numId()"/>
              <xsl:attribute name="id"><xsl:value-of select="$numid"/></xsl:attribute>
              <xsl:attribute name="lvl"><xsl:value-of select="$ilvl"/></xsl:attribute>
              <xsl:attribute name="class">word</xsl:attribute>
            </xsl:element>
          </xsl:if>
          <xsl:apply-templates/>
        </p>
	<xsl:if test="$defer_footnotes = 'true'">
        <xsl:for-each select="w:r/w:footnoteReference">
          <xsl:variable name="id" select="@w:id"/>
          <dbt:note>
	    <xsl:message>progress suspend</xsl:message>
            <xsl:apply-templates select="document('word/footnotes.xml')/w:footnotes/w:footnote[@w:id = $id]"/>
	    <xsl:message>progress resume</xsl:message>
          </dbt:note>
        </xsl:for-each>
	</xsl:if>
      </xsl:when>
      <xsl:when test="w:sectPr"/>
      <xsl:when test="w:r/w:object/o:OLEObject[@ProgID = 'Equation.DSMT4' or @ProgID = 'Equation.3']">
        <!-- Paragraph is empty except for embedded equation(s) -->
	<p xmlns="http://www.duxburysystems.com/dbtml" xmlns:dbt="http://www.duxburysystems.com/dbt"><xsl:apply-templates/></p>
      </xsl:when>
      <xsl:when test="w:r/w:pict/v:shape/v:textbox">
        <!-- Paragraph is empty except for text box -->
	<xsl:apply-templates/>
      </xsl:when>
      <xsl:when test="w:r/w:drawing">
        <!-- Paragraph is empty except for picture, which *should* have alt text -->
	<xsl:apply-templates/>
      </xsl:when>
      <xsl:when test="m:oMath">
        <!-- Paragraph is empty except for math, which can be transcribed -->
	<xsl:apply-templates/>
      </xsl:when>
      <xsl:otherwise>
        <!-- empty paragraph - make a blank line -->
        <xsl:if test="$psl = 'true'">
          <br/>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="simple-hardpage" match="w:br[@w:type = 'page'][../preceding-sibling::w:r[w:t]]">
    <xsl:if test="$phpb = 'true'">
      <dbt:hardpage/>
    </xsl:if>
    <xsl:if test="$dbt_version >= 110200 and $print_pages = 'scanned'">
      <!-- DBT 11.1 and earlier *will not process* dbt:scannedpagebreak -->
      <!-- DBT 11.2 and later should see hard page breaks marked with dbt:scannedpagebreak *only* for "scanned" documents -->
      <dbt:scannedpagebreak/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="w:br[not(@w:type)]">
    <dbt:l/>
  </xsl:template>

  <xsl:template name="sectPr">
    <xsl:param name="sect"/>
    <xsl:param name="pgbreak"/>
    <xsl:variable name="breaktype" select="$sect/w:type/@w:val"/>
    <xsl:if test="$pgbreak and not($breaktype and $breaktype = 'continuous') and ($phpb = 'true')">
      <xsl:element name="dbt:hardpage">
        <xsl:if test="$breaktype">
          <xsl:attribute name="type"><xsl:value-of select="$breaktype"/></xsl:attribute>
        </xsl:if>
      </xsl:element>
    </xsl:if>
    <xsl:if test="$sect/w:headerReference">
      <xsl:variable name="ridHeader" select="$sect/w:headerReference/@r:id"/>
      <xsl:variable name="runs" select="document(func:reldoc($ridHeader))/w:hdr/w:p/w:r[*]"/>
      <xsl:if test="$pgbreak or $runs">
        <dbt:tls/><xsl:for-each select="$runs"><xsl:value-of select="w:t"/></xsl:for-each><dbt:tle/>
      </xsl:if>
    </xsl:if>
    <xsl:if test="$sect/w:footerReference">
      <xsl:variable name="ridFooter" select="$sect/w:footerReference/@r:id"/>
      <xsl:variable name="runs" select="document(func:reldoc($ridFooter))/w:ftr/w:p/w:r[*]"/>
      <xsl:if test="$pgbreak or $runs">
        <dbt:rfs/><xsl:for-each select="$runs"><xsl:value-of select="w:t"/></xsl:for-each><dbt:rfe/>
      </xsl:if>
    </xsl:if>
  </xsl:template>

  <!-- I had the following, and a similar change in w:p, but I can't
       remember why, and it broke some tests, so I'm reverting. -MPG -->
  <!-- <xsl:template match="w:r[w:t|w:footnoteReference]"> -->
  <xsl:template match="w:r">
    <xsl:message>progress w:r</xsl:message>
    <xsl:if test="w:pict/v:shape/v:textbox">
      <box>
	<!-- Now get the text -->
	<xsl:apply-templates select="w:pict/v:shape/v:textbox/w:txbxContent/*"/>
      </box>
    </xsl:if>
    <xsl:if test="not (w:pict/v:shape/v:textbox)">
    <xsl:element name="text">

      <!-- Copy xml:space attribute from source to target. We must preserve spacing where Word does. -->
      <xsl:copy-of select="w:t/@*"/>

      <!-- Copy Word style to our "class" attribute -->
      <xsl:if test="w:rPr/w:rStyle and not (parent::w:hyperlink[starts-with(@w:anchor, '_Toc')])">
        <xsl:attribute name="class"><xsl:value-of select="w:rPr/w:rStyle/@w:val"/></xsl:attribute>
      </xsl:if>

      <!-- Copy Word language tag to our "lng" attribute -->
      <xsl:if test="$ignore_lang = 'false' and w:rPr/w:lang">
	<xsl:attribute name="lng"><xsl:value-of select="w:rPr/w:lang/@w:val"/></xsl:attribute>
      </xsl:if>
      
      <!-- Copy Word font name to our "font" attribute -->
      <xsl:if test="w:rPr/w:rFonts/@w:ascii">
	<xsl:attribute name="font"><xsl:value-of select="w:rPr/w:rFonts/@w:ascii"/></xsl:attribute>
      </xsl:if>

      <!-- Attributes (bold/italic/underline/subscript/superscript -->
      <xsl:call-template name="add-attributes"/>
      
      <!-- Now get the text -->
      <xsl:apply-templates/>
    </xsl:element>
    </xsl:if>
  </xsl:template>

  <!-- Footnote references (where they appear at the start of the footnote) -->
  <xsl:template match="w:footnoteRef">
    <!-- Write the footnote reference -->
    <xsl:value-of select="../../../@w:id"/>
  </xsl:template>

  <!-- Endnote references (where they appear at the start of the endnote) -->
  <xsl:template match="w:endnoteRef">
    <!-- Write the endnote reference -->
    <xsl:value-of select="../../../@w:id"/>
  </xsl:template>

  <xsl:template match="w:commentReference">
    <xsl:variable name="id" select="@w:id"/>
    <xsl:variable name="wholecomment" select="document('word/comments.xml')/w:comments/w:comment[@w:id = $id]/w:p/w:r/w:t"/>
    <xsl:variable name="commentstart" select="substring-before($wholecomment, ':')"/>
    <xsl:variable name="pagenumber" select="substring-after($wholecomment, ':')"/>
    <xsl:if test="$commentstart = 'DBTPageBreak'">
      <dbt:pagebreak><xsl:value-of select="$pagenumber"/></dbt:pagebreak>
    </xsl:if>
  </xsl:template>

  <!-- Footnote references (where they appear within the document body) -->
  <xsl:template match="w:footnoteReference">
    <!-- Write the footnote reference into document body -->
    <xsl:value-of select="@w:id"/>
    <xsl:if test="$defer_footnotes = 'false'">
      <!-- Write the footnote text right here in place. -->
      <xsl:variable name="id" select="@w:id"/>
      <dbt:note>
	<xsl:message>progress suspend</xsl:message>
	<xsl:apply-templates select="document('word/footnotes.xml')/w:footnotes/w:footnote[@w:id = $id]/w:p"/>
	<xsl:message>progress resume</xsl:message>
      </dbt:note>
    </xsl:if>
  </xsl:template>

  <!-- Endnote references (where they appear within the document body) -->
  <xsl:template match="w:endnoteReference">
    <!-- Write the endnote reference into document body -->
    <xsl:value-of select="@w:id"/>
    <xsl:if test="$defer_endnotes = 'false'">
      <!-- Write the endnote text right here in place. -->
      <xsl:variable name="id" select="@w:id"/>
      <dbt:note>
	<xsl:message>progress suspend</xsl:message>
	<xsl:apply-templates select="document('word/endnotes.xml')/w:endnotes/w:endnote[@w:id = $id]/w:p"/>
	<xsl:message>progress resume</xsl:message>
      </dbt:note>
    </xsl:if>
  </xsl:template>

  <xsl:template match="w:r/w:tab">
    <xsl:choose>
      <xsl:when test="ancestor::w:p/w:pPr/w:tabs/w:tab[@w:val = 'right'][@w:leader = 'dot']">
        <dbt:ltab/>
      </xsl:when>
      <xsl:otherwise>
        <dbt:tab/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="w:hyperlink">
    <xsl:variable name="ridHyperlink" select="@r:id"/>
    <xsl:variable name="rel" select="dbtfunc:wordrel($ridHyperlink)"/>
    <xsl:choose>
      <xsl:when test="$rel/@TargetMode = 'External'">
        <!-- For external hyperlinks, preserve URI and text in format that will be sorted out later, according to user's Global Hyperlinks selection -->
        <!-- See code in word2007.sbt and xmlproc/hyper.cpp -->
        <dbt:hyperlink-byte/><xsl:value-of select="$rel/@Target"/><dbt:hyperlink-byte/><xsl:apply-templates/><dbt:hyperlink-byte/>
      </xsl:when>
      <xsl:otherwise>
        <!-- For internal hyperlinks, e.g. within a table of contents, keep the text only (not the URI) -->
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Remove field metadata -->
  <xsl:template match="w:instrText"/>

  <!-- Remove anchor (positioning) metadata (typically controls placement of a picture) -->
  <xsl:template match="wp:anchor">
    <xsl:if test="wp:docPr[@descr]">
      <dbt:alttext><xsl:value-of select="wp:docPr/@descr"/></dbt:alttext>
    </xsl:if>
  </xsl:template>

  <!-- tables -->
  <xsl:template match="w:tbl">
    <!-- Insert htbs code with parameters as plain text, viz: <dbt:htbs>r:0:0:b:n:a</dbt:htbs>
	 word2007.sbt will recombine this to [htbs;r:0:0:b:n:a]. The approach is used because
         our parameter syntax is not XML-compliant. We want to include parameters so that we
	 can specify the header row count, as given in the XML markup for Word. -->
    <dbt:htbs>r:<xsl:value-of select="count(w:tr/w:trPr/w:tblHeader)"/>:0:b:n:a</dbt:htbs>
    <xsl:apply-templates/>
    <dbt:htbe/>
    <xsl:if test="$defer_footnotes = 'true'">
      <xsl:for-each select="w:tr/w:tc/w:p/w:r/w:footnoteReference">
	<xsl:variable name="id" select="@w:id"/>
	<dbt:note>
	  <xsl:message>progress suspend</xsl:message>
	  <xsl:apply-templates select="document('word/footnotes.xml')/w:footnotes/w:footnote[@w:id = $id]"/>
	  <xsl:message>progress resume</xsl:message>
	</dbt:note>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>
  <xsl:template match="w:tr">
    <xsl:if test="preceding-sibling::w:tr">
      <dbt:newline/>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="w:tc">
    <xsl:if test="preceding-sibling::w:tc">
      <dbt:tab/>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>

  <!-- math -->
  <xsl:template match="m:oMathPara">
    <dbt:ts/>
    <xsl:apply-templates/>
    <dbt:te/>
  </xsl:template>
  <xsl:template match="m:sup">
    <dbt:ps/>
    <xsl:apply-templates/>
    <dbt:pe/>
  </xsl:template>
  <!-- tbd - how is nth root handled? -->
  <xsl:template match="m:rad">
    <dbt:ixrts/>
    <xsl:apply-templates/>
    <dbt:ixrte/>
  </xsl:template>
  <xsl:template match="m:f">
    <dbt:fs/>
    <xsl:apply-templates select="m:num"/>
    <dbt:fl/>
    <xsl:apply-templates select="m:den"/>
    <dbt:fe/>
  </xsl:template>

  <xsl:template match="o:OLEObject[@ProgID = 'Equation.DSMT4']">
    <xsl:variable name="alttext" select="../v:shape/@alt"/>
    <xsl:variable name="ridDSMT" select="@r:id"/>
    <xsl:variable name="rel" select="document('word/_rels/document.xml.rels')/rp:Relationships/rp:Relationship[@Id = $ridDSMT]"/>
    <dbt:mathtype>
      <xsl:attribute name="stream"><xsl:value-of select="$rel/@Target"/></xsl:attribute>
    </dbt:mathtype>
    <xsl:if test="$alttext">
      <dbt:l/><xsl:value-of select="$alttext"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="o:OLEObject[@ProgID = 'Equation.3']">
    <xsl:variable name="alttext" select="../v:shape/@alt"/>
    <xsl:variable name="ridEquation" select="@r:id"/>
    <xsl:variable name="rel" select="document('word/_rels/document.xml.rels')/rp:Relationships/rp:Relationship[@Id = $ridEquation]"/>
    <dbt:equation>
      <xsl:attribute name="stream"><xsl:value-of select="$rel/@Target"/></xsl:attribute>
    </dbt:equation>
    <xsl:if test="$alttext">
      <dbt:l/><xsl:value-of select="$alttext"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="endnotes">
    <xsl:if test="$defer_endnotes = 'true'">
      <xsl:for-each select="w:p/w:r/w:endnoteReference">
	<xsl:variable name="id" select="@w:id"/>
	<dbt:note>
	  <xsl:message>progress suspend</xsl:message>
	  <xsl:apply-templates select="document('word/endnotes.xml')/w:endnotes/w:endnote[@w:id = $id]"/>
	  <xsl:message>progress resume</xsl:message>
	</dbt:note>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>

      <!-- Add "attribset" and "attribreset" attributes to current node, if w:Pr child has Word forms of same -->
  <xsl:template name="add-attributes">
    <xsl:variable name="char-setbold" select="substring('b', 1, $ignore_bold = 'false' and func:enabled(w:rPr/w:b))"/>
    <xsl:variable name="char-setital" select="substring('i', 1, $ignore_italics = 'false' and func:enabled(w:rPr/w:i))"/>
    <xsl:variable name="char-setund" select="substring('u', 1, $ignore_underline = 'false' and func:enabled(w:rPr/w:u))"/>
    <xsl:variable name="char-setftss" select="substring('c', 1, $ignore_typeforms = 'false' and func:ftss())"/> <!-- Using c for "cursive", meaning script -->
    <xsl:variable name="char-setfts1" select="substring('1', 1, $ignore_typeforms = 'false' and func:fts1())"/>
    <xsl:variable name="char-setfts2" select="substring('2', 1, $ignore_typeforms = 'false' and func:fts2())"/>
    <xsl:variable name="char-setfts3" select="substring('3', 1, $ignore_typeforms = 'false' and func:fts3())"/>
    <xsl:variable name="char-setfts4" select="substring('4', 1, $ignore_typeforms = 'false' and func:fts4())"/>
    <xsl:variable name="char-setfts5" select="substring('5', 1, $ignore_typeforms = 'false' and func:fts5())"/>
    <xsl:variable name="char-setsuper" select="substring('S', 1, w:rPr/w:vertAlign/@w:val = 'superscript')"/>
    <xsl:variable name="char-setsub" select="substring('s', 1, w:rPr/w:vertAlign/@w:val = 'subscript')"/>
    <xsl:variable name="char-setattrib" select="concat($char-setbold, $char-setital, $char-setund, $char-setsuper, $char-setsub, $char-setftss, $char-setfts1, $char-setfts2, $char-setfts3, $char-setfts4, $char-setfts5)"/>
    <xsl:if test="$char-setattrib">
      <xsl:attribute name="attribset"><xsl:value-of select="$char-setattrib"/></xsl:attribute>
    </xsl:if>

    <xsl:variable name="char-resetbold" select="substring('b', 1, $ignore_bold = 'false' and func:disabled(w:rPr/w:b))"/>
    <xsl:variable name="char-resetital" select="substring('i', 1, $ignore_italics = 'false' and func:disabled(w:rPr/w:i))"/>
    <xsl:variable name="char-resetund" select="substring('u', 1, $ignore_underline = 'false' and func:disabled(w:rPr/w:u))"/>
    <xsl:variable name="char-resetftss" select="substring('c', 1, $ignore_typeforms = 'false' and not (func:ftss()))"/> <!-- Using c for "cursive", meaning script -->
    <xsl:variable name="char-resetfts1" select="substring('1', 1, $ignore_typeforms = 'false' and not (func:fts1()))"/>
    <xsl:variable name="char-resetfts2" select="substring('2', 1, $ignore_typeforms = 'false' and not (func:fts2()))"/>
    <xsl:variable name="char-resetfts3" select="substring('3', 1, $ignore_typeforms = 'false' and not (func:fts3()))"/>
    <xsl:variable name="char-resetfts4" select="substring('4', 1, $ignore_typeforms = 'false' and not (func:fts4()))"/>
    <xsl:variable name="char-resetfts5" select="substring('5', 1, $ignore_typeforms = 'false' and not (func:fts5()))"/>
    <xsl:variable name="char-resetsuper" select="substring('S', 1, w:rPr/w:vertAlign/@w:val and not (w:rPr/w:vertAlign/@w:val = 'superscript'))"/>
    <xsl:variable name="char-resetsub" select="substring('s', 1, w:rPr/w:vertAlign/@w:val and not (w:rPr/w:vertAlign/@w:val = 'subscript'))"/>
    <xsl:variable name="char-resetattrib" select="concat($char-resetbold, $char-resetital, $char-resetund, $char-resetsuper, $char-resetsub, $char-resetftss, $char-resetfts1, $char-resetfts2, $char-resetfts3, $char-resetfts4, $char-resetfts5)"/>
    <xsl:if test="$char-resetattrib">
      <xsl:attribute name="attribreset"><xsl:value-of select="$char-resetattrib"/></xsl:attribute>
    </xsl:if>
  </xsl:template>

  <!-- Ad-hoc handling of checked/unchecked checkboxes. Useful for forms. -->
  <xsl:template match="w:checkBox">
    <dbt:checkbox>
      <xsl:choose>
        <xsl:when test="w:checked">(*)</xsl:when>
	<xsl:when test="w:default/@w:val = 1">(*)</xsl:when>
        <xsl:otherwise>( )</xsl:otherwise>
      </xsl:choose>
    </dbt:checkbox>
  </xsl:template>

  <!-- picture alt text -->
  <xsl:template match="wp:docPr[@descr]">
    <dbt:alttext><xsl:value-of select="@descr"/></dbt:alttext>
  </xsl:template>
</xsl:stylesheet>
